from django.db import models
from modelcluster.fields import ParentalKey
from wagtail.wagtailadmin.edit_handlers import (
    FieldPanel, MultiFieldPanel, PageChooserPanel)
from wagtailmenus.models import (
    ChildrenMenu, SectionMenu, AbstractMainMenu, AbstractMainMenuItem,
    AbstractFlatMenu, AbstractFlatMenuItem)

from .utils import TranslatedField


class CustomChildrenMenu(ChildrenMenu):
    pass


class CustomSectionMenu(SectionMenu):
    pass


class MultilingualMenuItem(models.Model):
    link_text_de = models.CharField(
        verbose_name='link text (de)',
        max_length=255,
        blank=True,
    )
    link_text_fr = models.CharField(
        verbose_name='link text (fr)',
        max_length=255,
        blank=True,
    )
    translated_link_text = TranslatedField(
        'link_text', 'link_text_de', 'link_text_fr'
    )

    class Meta:
        abstract = True
        ordering = ('sort_order',)

    @property
    def menu_text(self):
        return self.translated_link_text or getattr(
            self.link_page, 'translated_title', None
        ) or self.link_page.title

    panels = (
        PageChooserPanel('link_page'),
        FieldPanel('link_url'),
        FieldPanel('url_append'),
        FieldPanel('link_text'),
        FieldPanel('link_text_de'),
        FieldPanel('link_text_fr'),
        FieldPanel('handle'),
        FieldPanel('allow_subnav'),
    )


class MainMenuCustomMenuItem(MultilingualMenuItem, AbstractMainMenuItem):
    """Custom MenuItem model for the default MainMenu model. The default
    model is swapped out for this one using the setting:

    `WAGTAILMENUS_MAIN_MENU_ITEMS_RELATED_NAME = 'custom_menu_items'
    """
    menu = ParentalKey(
        'wagtailmenus.MainMenu',
        on_delete=models.CASCADE,
        related_name="custom_menu_items"
    )


class FlatMenuCustomMenuItem(MultilingualMenuItem, AbstractFlatMenuItem):
    """Custom MenuItem model for the default FlatMenu model. The default
    model is swapped out for this one using the setting:

    `WAGTAILMENUS_FLAT_MENU_ITEMS_RELATED_NAME = 'custom_menu_items'
    """

    menu = ParentalKey(
        'wagtailmenus.FlatMenu',
        on_delete=models.CASCADE,
        related_name="custom_menu_items"
    )


class CustomMainMenu(AbstractMainMenu):
    pass


class CustomFlatMenu(AbstractFlatMenu):
    heading_de = models.CharField(
        verbose_name='heading (de)',
        max_length=255,
        blank=True,
    )
    heading_fr = models.CharField(
        verbose_name='heading (fr)',
        max_length=255,
        blank=True,
    )
    translated_heading = TranslatedField(
        'heading', 'heading_de', 'heading_fr'
    )

    panels = (
        MultiFieldPanel(
            heading="Settings",
            children=(
                FieldPanel('title'),
                FieldPanel('site'),
                FieldPanel('handle'),
            )
        ),
        MultiFieldPanel(
            heading="Heading",
            children=(
                FieldPanel('heading'),
                FieldPanel('heading_de'),
                FieldPanel('heading_fr'),
            ),
            classname='collapsible'
        ),
        AbstractFlatMenu.panels[1],
        AbstractFlatMenu.panels[2],
    )


class CustomMainMenuItem(MultilingualMenuItem, AbstractMainMenuItem):
    """Custom MenuItem model for `CustomMainMenu`. Notice the `related_name`
    attribue on the field below is the same as it is on
    wagtailmenus.MainMenuItem. Because of this, the
    `WAGTAILMENUS_MAIN_MENU_ITEMS_RELATED_NAME` setting doesn't need to be
    overridden ('menu_items' is the default value)."""

    menu = ParentalKey(
        'CustomMainMenu',
        on_delete=models.CASCADE,
        related_name="menu_items"
    )


class CustomFlatMenuItem(MultilingualMenuItem, AbstractFlatMenuItem):
    """Custom MenuItem model for `CustomFlatMenu`. Notice the `related_name`
    attribue on the field below is the same as it is on
    wagtailmenus.FlatMenuItem. Because of this, the
    `WAGTAILMENUS_FLAT_MENU_ITEMS_RELATED_NAME` setting doesn't need to be
    overridden ('menu_items' is the default value)."""

    menu = ParentalKey(
        'CustomFlatMenu',
        on_delete=models.CASCADE,
        related_name="menu_items"
    )



from __future__ import absolute_import, unicode_literals

from django.test import TestCase
from django.core.exceptions import ValidationError

from wagtail.wagtailcore.models import Page, Site
from wagtailmenus.models import MainMenu, MainMenuItem, FlatMenu
from wagtailmenus.tests.models import LinkPage


class TestModels(TestCase):
    fixtures = ['test.json']

    def test_mainmenuitem_clean_missing_link_text(self):
        menu = MainMenu.objects.get(pk=1)
        new_item = MainMenuItem(menu=menu, link_url='test/')
        self.assertRaisesMessage(
            ValidationError,
            "This field is required when linking to a custom URL",
            new_item.clean)

    def test_mainmenuitem_clean_missing_link_url(self):
        menu = MainMenu.objects.get(pk=1)
        new_item = MainMenuItem(menu=menu)
        self.assertRaisesMessage(
            ValidationError,
            "Please choose an internal page or provide a custom URL",
            new_item.clean)

    def test_mainmenuitem_clean_link_url_and_link_page(self):
        menu = MainMenu.objects.get(pk=1)
        new_item = MainMenuItem(
            menu=menu,
            link_text='Test',
            link_url='test/',
            link_page=Page.objects.get(pk=6))
        self.assertRaisesMessage(
            ValidationError,
            "Linking to both a page and custom URL is not permitted",
            new_item.clean)

    def test_mainmenuitem_str(self):
        menu = MainMenu.objects.get(pk=1)
        item_1 = menu.menu_items.first()
        self.assertEqual(item_1.__str__(), 'Home')

    def test_flatmenuitem_str(self):
        menu = FlatMenu.objects.get(handle='contact')
        item_1 = menu.menu_items.first()
        self.assertEqual(item_1.__str__(), 'Call us')

    def test_mainmenu_top_level_items(self):
        menu = MainMenu.objects.get(pk=1)
        # This menu has a `use_specific` value of 1 (AUTO)
        self.assertEqual(menu.use_specific, 1)

        # So, the top-level items it return should all just be custom links
        # or have a `link_page` that is just a vanilla Page object.
        for item in menu.top_level_items:
            self.assertTrue(item.link_page is None or type(item.link_page) is Page)

        # After being called once, top_level_options should be cached, so
        # accessing it again shouldn't trigger any database queries
        with self.assertNumQueries(0):
            for item in menu.top_level_items:
                self.assertTrue(item.link_page or item.link_url)

        # Lets change `use_specific` to 2 (TOP_LEVEL)
        menu.set_use_specific(2)

        # The above should clear the cached value, so if we call
        # top_level_items again, we should get a fresh list of specific pages
        for item in menu.top_level_items:
            self.assertTrue(item.link_page is None or type(item.link_page) is not Page)

        # Lets change `use_specific` to 0 (OFF) again
        menu.set_use_specific(0)

        # Because the work has already been done to fetch specific items, even
        # though we might not need specific items, the cached value should
        # remain in-tact, with no more queries being triggered on recall.
        with self.assertNumQueries(0):
            for item in menu.top_level_items:
                assert item.link_page is None or type(item.link_page) is not Page

    def test_mainmenu_pages_for_display(self):
        menu = MainMenu.objects.get(pk=1)
        # This menu has a `use_specific` value of 1 (AUTO)
        self.assertEqual(menu.use_specific, 1)
        # And a `max_levels` value of 2
        self.assertEqual(menu.max_levels, 2)

        # So, every page returned by `pages_for_display` should just be a
        # vanilla Page object, that is live, not expired and meant to
        # appear in menus
        for p in menu.pages_for_display:
            self.assertEqual(type(p), Page)
            self.assertTrue(p.live)
            self.assertFalse(p.expired)
            self.assertTrue(p.show_in_menus)

        # Their should be 6 pages total
        self.assertEqual(len(menu.pages_for_display), 6)

        # After being called once, pages_for_display should be cached, so
        # accessing it again shouldn't trigger any database queries
        with self.assertNumQueries(0):
            for p in menu.pages_for_display:
                self.assertTrue(p.title)

        # Lets change `use_specific` to 3 (ALWAYS)
        menu.set_use_specific(3)

        # The above should clear the cached value, so if we call
        # pages_for_display again, we should get a fresh list of specific pages
        for p in menu.pages_for_display:
            self.assertNotEqual(type(p), Page)

        # Lets change `max_levels` to 1
        menu.set_max_levels(1)

        # The above should clear the cached value, so if we call
        # pages_for_display again, we it should provide an empty queryset
        self.assertQuerysetEqual(menu.pages_for_display, Page.objects.none())

        # Lets change `max_levels` to 3
        menu.set_max_levels(3)

        # The above should cleare the cached value, and now pages_for_display
        # should reurn 12 pages total to display
        self.assertEqual(len(menu.pages_for_display), 9)

        # As with `top_level_items` Even is we set `use_specific` to a lower
        # value, the cached `pages_for_display` value will not be cleared, as
        # it specific is better than not.
        menu.set_use_specific(1)

        # Because the work has already been done to fetch specific pages, even
        # though we might not need specific items, the cached value should
        # remain in-tact, with no more queries being triggered on recall.
        with self.assertNumQueries(0):
            for p in menu.pages_for_display:
                assert type(p) is not Page


class TestLinkPage(TestCase):
    fixtures = ['test.json']

    def setUp(self):
        # Create a few of link pages for testing
        site = Site.objects.select_related('root_page').get(is_default_site=True)
        self.site = site

        linkpage_to_page = LinkPage(
            title='Find out about Spiderman',
            link_page_id=30,
            url_append='?somevar=value'
        )
        site.root_page.add_child(instance=linkpage_to_page)

        # Check that the above page was saved and has field values we expect
        self.assertTrue(linkpage_to_page.id)
        self.assertTrue(linkpage_to_page.show_in_menus)
        self.assertTrue(linkpage_to_page.show_in_menus_custom())
        self.assertEqual(linkpage_to_page.get_sitemap_urls(), [])
        self.linkpage_to_page = linkpage_to_page

        linkpage_to_url = LinkPage(
            title='Do a google search',
            link_url="https://www.google.co.uk",
            url_append='?somevar=value',
            extra_classes='google external',
        )
        site.root_page.add_child(instance=linkpage_to_url)

        # Check that the above page was saved and has field values we expect
        self.assertTrue(linkpage_to_url.id)
        self.assertTrue(linkpage_to_url.show_in_menus)
        self.assertTrue(linkpage_to_url.show_in_menus_custom())
        self.assertEqual(linkpage_to_url.get_sitemap_urls(), [])
        self.linkpage_to_url = linkpage_to_url

        linkpage_to_non_routable_page = LinkPage(
            title='Go to this unroutable page',
            link_page_id=2,
            url_append='?somevar=value'
        )
        site.root_page.add_child(instance=linkpage_to_non_routable_page)
        self.linkpage_to_non_routable_page = linkpage_to_non_routable_page

    def test_url_methods(self):
        # When linking to a page
        self.assertEqual(
            self.linkpage_to_page.relative_url(self.site),
            '/superheroes/marvel-comics/spiderman/?somevar=value'
        )
        self.assertEqual(
            self.linkpage_to_page.full_url,
            'http://www.wagtailmenus.co.uk:8000/superheroes/marvel-comics/spiderman/?somevar=value'
        )

        # When linking to a non-routable page
        self.assertEqual(self.linkpage_to_non_routable_page.relative_url(self.site), '')
        self.assertEqual(self.linkpage_to_non_routable_page.full_url, '')

        # When linking to a custom url
        self.assertEqual(
            self.linkpage_to_url.relative_url(self.site), 'https://www.google.co.uk?somevar=value'
        )
        self.assertEqual(
            self.linkpage_to_url.full_url, 'https://www.google.co.uk?somevar=value'
        )

    def test_linkpage_visibility(self):
        page_link_html = (
            '<a href="/superheroes/marvel-comics/spiderman/?somevar=value">Find out about Spiderman</a>'
        )

        url_link_html = (
            '<li class="google external"><a href="https://www.google.co.uk?somevar=value">Do a google search</a></li>'
        )
        # When the target page is live, both the 'Spiderman' and 'Google' link should appear 
        response = self.client.get('/')
        self.assertContains(response, page_link_html, html=True)
        self.assertContains(response, url_link_html, html=True)

        # When the target page is not live, the linkpage shouldn't appear
        target_page = self.linkpage_to_page.link_page
        target_page.live = False
        target_page.save()
        response = self.client.get('/')
        self.assertNotContains(response, page_link_html, html=True)

        # When the target page isn't set to appear in menus, the linkpage
        # shouldn't appear
        target_page.live = True
        target_page.show_in_menus = False
        target_page.save()
        response = self.client.get('/')
        self.assertNotContains(response, page_link_html, html=True)

        # When the target page is 'expired', the linkpage shouldn't appear
        target_page.show_in_menus = True
        target_page.expired = True
        target_page.save()
        response = self.client.get('/')
        self.assertNotContains(response, page_link_html, html=True)

    def test_linkpage_clean(self):
        linkpage = self.linkpage_to_page
        linkpage.link_url = 'https://www.rkh.co.uk/'
        self.assertRaisesMessage(
            ValidationError,
            "Linking to both a page and custom URL is not permitted",
            linkpage.clean
        )

        linkpage.link_url = ''
        linkpage.link_page = None
        self.assertRaisesMessage(
            ValidationError,
            "Please choose an internal page or provide a custom URL",
            linkpage.clean
        )

        linkpage.link_page = linkpage
        self.assertRaisesMessage(
            ValidationError,
            "A link page cannot link to another link page",
            linkpage.clean
        )

    def test_linkpage_redirects_when_served(self):
        response = self.client.get('/find-out-about-spiderman/')
        self.assertRedirects(
            response,
            '/superheroes/marvel-comics/spiderman/?somevar=value'
        )



# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2017-01-06 23:52
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailmenus', '0020_auto_20161210_0004'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='flatmenuitem',
            options={'ordering': ('sort_order',)},
        ),
        migrations.AlterModelOptions(
            name='mainmenuitem',
            options={'ordering': ('sort_order',)},
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2016-10-13 15:58
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailmenus', '0016_auto_20160930_0645'),
    ]

    operations = [
        migrations.AddField(
            model_name='flatmenu',
            name='max_levels',
            field=models.PositiveSmallIntegerField(choices=[(1, '1: No sub-navigation (flat)'), (2, '2: Allow 1 level of sub-navigation'), (3, '3: Allow 2 levels of sub-navigation'), (4, '4: Allow 3 levels of sub-navigation')], default=1, help_text='The maximum number of levels to display when rendering this menu. The value can be overidden by supplying a different <code>max_levels</code> value to the <code>{% flat_menu %}</code> tag in your templates.', verbose_name='maximum levels'),
        ),
        migrations.AddField(
            model_name='flatmenu',
            name='use_specific',
            field=models.PositiveSmallIntegerField(choices=[(0, 'OFF (Most efficient)'), (1, 'AUTO'), (2, 'TOP_LEVEL'), (3, 'ALWAYS (Least efficient)')], default=1, help_text="Controls how 'specific' pages objects are fetched and used when rendering this menu. This value can be overidden by supplying a different <code>use_specific</code> value to the <code>{% flat_menu %}</code> tag in your templates.", verbose_name='specific page usage'),
        ),
        migrations.AddField(
            model_name='mainmenu',
            name='max_levels',
            field=models.PositiveSmallIntegerField(choices=[(1, '1: No sub-navigation (flat)'), (2, '2: Allow 1 level of sub-navigation'), (3, '3: Allow 2 levels of sub-navigation'), (4, '4: Allow 3 levels of sub-navigation')], default=2, help_text='The maximum number of levels to display when rendering this menu. The value can be overidden by supplying a different <code>max_levels</code> value to the <code>{% main_menu %}</code> tag in your templates.', verbose_name='maximum levels'),
        ),
        migrations.AddField(
            model_name='mainmenu',
            name='use_specific',
            field=models.PositiveSmallIntegerField(choices=[(0, 'OFF (Most efficient)'), (1, 'AUTO'), (2, 'TOP_LEVEL'), (3, 'ALWAYS (Least efficient)')], default=1, help_text="Controls how 'specific' pages objects are fetched and used when rendering this menu. This value can be overidden by supplying a different <code>use_specific</code> value to the <code>{% main_menu %}</code> tag in your templates.", verbose_name='specific page usage'),
        ),
        migrations.AlterField(
            model_name='mainmenu',
            name='site',
            field=models.OneToOneField(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='main_menu', to='wagtailcore.Site'),
        ),
    ]



from __future__ import absolute_import, unicode_literals

from collections import defaultdict

from django.db import models
from django.core.exceptions import ImproperlyConfigured, ValidationError
from django.utils.encoding import python_2_unicode_compatible
from django.utils.functional import cached_property
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

from modelcluster.models import ClusterableModel

from wagtail.wagtailadmin.edit_handlers import (
    FieldPanel, MultiFieldPanel, InlinePanel)
from wagtail.wagtailcore.models import Page

from .. import app_settings
from ..forms import FlatMenuAdminForm


# ########################################################
# Base classes
# ########################################################

class Menu(object):
    """A base class that all other 'menu' classes should inherit from."""

    max_levels = 1
    use_specific = app_settings.USE_SPECIFIC_AUTO
    pages_for_display = None
    request = None

    def clear_page_cache(self):
        try:
            del self.pages_for_display
        except AttributeError:
            pass
        try:
            del self.page_children_dict
        except AttributeError:
            pass

    def get_base_page_queryset(self):
        return Page.objects.filter(live=True, expired=False,
                                   show_in_menus=True)

    def set_max_levels(self, max_levels):
        if self.max_levels != max_levels:
            """
            Set `self.max_levels` to the supplied value and clear any cached
            attribute values set for a different `max_levels` value.
            """
            self.max_levels = max_levels
            self.clear_page_cache()

    def set_use_specific(self, use_specific):
        if self.use_specific != use_specific:
            """
            Set `self.use_specific` to the supplied value and clear some
            cached values where appropriate.
            """
            if(
                use_specific >= app_settings.USE_SPECIFIC_TOP_LEVEL and
                self.use_specific < app_settings.USE_SPECIFIC_TOP_LEVEL
            ):
                self.clear_page_cache()
                try:
                    del self.top_level_items
                except AttributeError:
                    pass

            self.use_specific = use_specific

    def set_request(self, request):
        """
        Set `self.request` to the supplied HttpRequest, so that developers can
        make use of it in subclasses
        """
        self.request = request

    @property
    def pages_for_display(self):
        raise NotImplementedError("Subclasses of `Menu` must define their own "
                                  "'pages_for_display' method")

    @cached_property
    def page_children_dict(self):
        """Returns a dictionary of lists, where the keys are 'path' values for
        pages, and the value is a list of children pages for that page."""
        children_dict = defaultdict(list)
        for page in self.pages_for_display:
            children_dict[page.path[:-page.steplen]].append(page)
        return children_dict

    def get_children_for_page(self, page):
        """Return a list of relevant child pages for a given page."""
        return self.page_children_dict.get(page.path, [])

    def page_has_children(self, page):
        """Return a boolean indicating whether a given page has any relevant
        child pages."""
        return page.path in self.page_children_dict


class MenuFromRootPage(Menu):
    """A 'menu' that is instantiated with a 'root page', and whose 'menu items'
    consist solely of ancestors of that page."""

    root_page = None

    def __init__(self, root_page, max_levels, use_specific):
        self.root_page = root_page
        self.max_levels = max_levels
        self.use_specific = use_specific
        super(MenuFromRootPage, self).__init__()

    @cached_property
    def pages_for_display(self):
        """Returns a list of pages for rendering all levels of the menu. All
        pages must be live, not expired, and set to show in menus."""
        pages = self.get_base_page_queryset().filter(
            depth__gt=self.root_page.depth,
            depth__lte=self.root_page.depth + self.max_levels,
            path__startswith=self.root_page.path,
        )

        # Return 'specific' page instances if required
        if self.use_specific == app_settings.USE_SPECIFIC_ALWAYS:
            return pages.specific()

        return pages

    def get_children_for_page(self, page):
        """Return a list of relevant child pages for a given page."""
        if self.max_levels == 1:
            # If there's only a single level of pages to display, skip the
            # dict creation / lookup and just return the QuerySet result
            return self.pages_for_display
        return super(MenuFromRootPage, self).get_children_for_page(page)


class SectionMenu(MenuFromRootPage):
    pass


class ChildrenMenu(MenuFromRootPage):
    pass


class MenuWithMenuItems(ClusterableModel, Menu):
    """A base model class for menus who's 'menu_items' are defined by
    a set of 'menu item' model instances."""

    class Meta:
        abstract = True

    def get_base_menuitem_queryset(self):
        return self.get_menu_items_manager().for_display()

    @cached_property
    def top_level_items(self):
        """Return a list of menu items with link_page objects supplemented with
        'specific' pages where appropriate."""
        items_qs = self.get_base_menuitem_queryset()
        if self.use_specific < app_settings.USE_SPECIFIC_TOP_LEVEL:
            return items_qs.all()

        """
        The menu is being generated with a specificity level of TOP_LEVEL
        or ALWAYS, which means we need to replace 'link_page' values on
        MenuItem objects with their 'specific' equivalents.
        """
        updated_items = []
        for item in items_qs.all():
            if item.link_page_id:
                item.link_page = item.link_page.specific
            updated_items.append(item)
        return updated_items

    @cached_property
    def pages_for_display(self):
        """Return a list of pages for rendering the entire menu (excluding
        those chosen as menu items). All pages must be live, not expired, and
        set to show in menus."""

        # Build a queryset to get pages for all levels
        all_pages = Page.objects.none()

        if self.max_levels == 1:
            # If no additional menus are needed, return an empty queryset
            return all_pages

        for item in self.top_level_items:

            if item.link_page_id:
                # If necessary, fetch a 'branch' of suitable descendants for
                # this menu item and add to the full queryset
                page_path = item.link_page.path
                page_depth = item.link_page.depth
                if(
                    item.allow_subnav and
                    page_depth >= app_settings.SECTION_ROOT_DEPTH
                ):
                    all_pages = all_pages | Page.objects.filter(
                        depth__gt=page_depth,
                        depth__lt=page_depth + self.max_levels,
                        path__startswith=page_path)

        # Filter the queryset to include only the pages we need for display
        all_pages = all_pages & self.get_base_page_queryset()

        # Return 'specific' page instances if required
        if self.use_specific == app_settings.USE_SPECIFIC_ALWAYS:
            return all_pages.specific()

        return all_pages


# ########################################################
# Abstract models
# ########################################################

@python_2_unicode_compatible
class AbstractMainMenu(MenuWithMenuItems):
    site = models.OneToOneField(
        'wagtailcore.Site',
        verbose_name=_('site'),
        db_index=True,
        editable=False,
        on_delete=models.CASCADE,
        related_name="+",
    )
    max_levels = models.PositiveSmallIntegerField(
        verbose_name=_('maximum levels'),
        choices=app_settings.MAX_LEVELS_CHOICES,
        default=2,
        help_text=mark_safe(_(
            "The maximum number of levels to display when rendering this "
            "menu. The value can be overidden by supplying a different "
            "<code>max_levels</code> value to the <code>{% main_menu %}"
            "</code> tag in your templates."
        ))
    )
    use_specific = models.PositiveSmallIntegerField(
        verbose_name=_('specific page usage'),
        choices=app_settings.USE_SPECIFIC_CHOICES,
        default=app_settings.USE_SPECIFIC_AUTO,
        help_text=mark_safe(_(
            "Controls how 'specific' pages objects are fetched and used when "
            "rendering this menu. This value can be overidden by supplying a "
            "different <code>use_specific</code> value to the <code>"
            "{% main_menu %}</code> tag in your templates."
        ))
    )

    class Meta:
        abstract = True
        verbose_name = _("main menu")
        verbose_name_plural = _("main menu")

    @classmethod
    def get_for_site(cls, site):
        """Get a mainmenu instance for the site."""
        instance, created = cls.objects.get_or_create(site=site)
        return instance

    def __str__(self):
        return _('Main menu for %(site_name)s') % {
            'site_name': self.site.site_name or self.site
        }

    def get_menu_items_manager(self):
        try:
            return getattr(self, app_settings.MAIN_MENU_ITEMS_RELATED_NAME)
        except AttributeError:
            raise ImproperlyConfigured(
                "'%s' isn't a valid relationship name for accessing menu "
                "items from %s. Check that your "
                "`WAGTAILMENUS_MAIN_MENU_ITEMS_RELATED_NAME` setting matches "
                "the `related_name` used on your MenuItem model's "
                "`ParentalKey` field." % (
                    app_settings.MAIN_MENU_ITEMS_RELATED_NAME,
                    self.__class__.__name__
                )
            )

    panels = (
        InlinePanel(
            app_settings.MAIN_MENU_ITEMS_RELATED_NAME, label=_("menu items")
        ),
        MultiFieldPanel(
            heading=_("Advanced settings"),
            children=(FieldPanel('max_levels'), FieldPanel('use_specific')),
            classname="collapsible collapsed",
        ),
    )


@python_2_unicode_compatible
class AbstractFlatMenu(MenuWithMenuItems):
    site = models.ForeignKey(
        'wagtailcore.Site',
        verbose_name=_('site'),
        db_index=True,
        on_delete=models.CASCADE,
        related_name='+'
    )
    title = models.CharField(
        verbose_name=_('title'),
        max_length=255,
        help_text=_("For internal reference only.")
    )
    handle = models.SlugField(
        verbose_name=_('handle'),
        max_length=100,
        help_text=_(
            "Used to reference this menu in templates etc. Must be unique "
            "for the selected site."
        )
    )
    heading = models.CharField(
        verbose_name=_('heading'),
        max_length=255,
        blank=True,
        help_text=_("If supplied, appears above the menu when rendered.")
    )
    max_levels = models.PositiveSmallIntegerField(
        verbose_name=_('maximum levels'),
        choices=app_settings.MAX_LEVELS_CHOICES,
        default=1,
        help_text=mark_safe(_(
            "The maximum number of levels to display when rendering this "
            "menu. The value can be overidden by supplying a different "
            "<code>max_levels</code> value to the <code>{% flat_menu %}"
            "</code> tag in your templates."
        ))
    )
    use_specific = models.PositiveSmallIntegerField(
        verbose_name=_('specific page usage'),
        choices=app_settings.USE_SPECIFIC_CHOICES,
        default=app_settings.USE_SPECIFIC_AUTO,
        help_text=mark_safe(_(
            "Controls how 'specific' pages objects are fetched and used when "
            "rendering this menu. This value can be overidden by supplying a "
            "different <code>use_specific</code> value to the <code>"
            "{% flat_menu %}</code> tag in your templates."
        ))
    )

    base_form_class = FlatMenuAdminForm

    class Meta:
        abstract = True
        unique_together = ("site", "handle")
        verbose_name = _("flat menu")
        verbose_name_plural = _("flat menus")

    @classmethod
    def get_for_site(cls, handle, site,
                     fall_back_to_default_site_menus=False):
        """Get a FlatMenu instance with a matching `handle` for the `site`
        provided - or for the 'default' site if not found."""
        menu = cls.objects.filter(handle__exact=handle, site=site).first()
        if(
            menu is None and fall_back_to_default_site_menus and
            not site.is_default_site
        ):
            return cls.objects.filter(
                handle__exact=handle, site__is_default_site=True
            ).first()
        return menu

    def __str__(self):
        return '%s (%s)' % (self.title, self.handle)

    def get_menu_items_manager(self):
        try:
            return getattr(self, app_settings.FLAT_MENU_ITEMS_RELATED_NAME)
        except AttributeError:
            raise ImproperlyConfigured(
                "'%s' isn't a valid relationship name for accessing menu "
                "items from %s. Check that your "
                "`WAGTAILMENUS_FLAT_MENU_ITEMS_RELATED_NAME` setting matches "
                "the `related_name` used on your MenuItem model's "
                "`ParentalKey` field." % (
                    app_settings.FLAT_MENU_ITEMS_RELATED_NAME,
                    self.__class__.__name__
                )
            )

    def get_template_names(self, request, template_name=None):
        """Returns a list of template names to search for when rendering this
        menu to a template. The first template that is found will be used."""

        if template_name:
            return [template_name]
        template_names = []
        if app_settings.SITE_SPECIFIC_TEMPLATE_DIRS and getattr(
            request, 'site', None
        ):
            hostname = request.site.hostname
            template_names.extend([
                "menus/%s/flat/%s/menu.html" % (hostname, self.handle,),
                "menus/%s/flat/%s.html" % (hostname, self.handle,),
                "menus/%s/%s/menu.html" % (hostname, self.handle,),
                "menus/%s/%s.html" % (hostname, self.handle,),
                "menus/%s/flat/menu.html" % hostname,
                "menus/%s/flat/default.html" % hostname,
                "menus/%s/flat_menu.html" % hostname,
            ])
        template_names.extend([
            "menus/flat/%s/menu.html" % self.handle,
            "menus/flat/%s.html" % self.handle,
            "menus/%s/menu.html" % self.handle,
            "menus/%s.html" % self.handle,
            "menus/flat/default.html",
            "menus/flat/menu.html",
            app_settings.DEFAULT_FLAT_MENU_TEMPLATE,
        ])
        return template_names

    def get_sub_menu_template_names(self, request, template_name=None):
        """Returns a list of template names to search for when rendering a
        sub menu for this menu to a template. The first template that is found
        will be used."""

        if template_name:
            return [template_name]
        template_names = []
        if app_settings.SITE_SPECIFIC_TEMPLATE_DIRS and getattr(
            request, 'site', None
        ):
            hostname = request.site.hostname
            template_names.extend([
                "menus/%s/flat/%s/sub_menu.html" % (hostname, self.handle,),
                "menus/%s/flat/%s_sub_menu.html" % (hostname, self.handle,),
                "menus/%s/%s/sub_menu.html" % (hostname, self.handle,),
                "menus/%s/%s_sub_menu.html" % (hostname, self.handle,),
                "menus/%s/flat/sub_menu.html" % hostname,
                "menus/%s/sub_menu.html" % hostname,
            ])
        template_names.extend([
            "menus/flat/%s/sub_menu.html" % self.handle,
            "menus/flat/%s_sub_menu.html" % self.handle,
            "menus/%s/sub_menu.html" % self.handle,
            "menus/%s_sub_menu.html" % self.handle,
            "menus/flat/sub_menu.html",
            app_settings.DEFAULT_SUB_MENU_TEMPLATE,
        ])
        return template_names

    def clean(self, *args, **kwargs):
        """Raise validation error for unique_together constraint, as it's not
        currently handled properly by wagtail."""

        clashes = self.__class__.objects.filter(site=self.site,
                                                handle=self.handle)
        if self.pk:
            clashes = clashes.exclude(pk__exact=self.pk)
        if clashes.exists():
            msg = _("Site and handle must create a unique combination. A menu "
                    "already exists with these same two values.")
            raise ValidationError({
                'site': [msg],
                'handle': [msg],
            })
        super(AbstractFlatMenu, self).clean(*args, **kwargs)

    panels = (
        MultiFieldPanel(
            heading=_("Settings"),
            children=(
                FieldPanel('title'),
                FieldPanel('site'),
                FieldPanel('handle'),
                FieldPanel('heading'),
            )
        ),
        InlinePanel(
            app_settings.FLAT_MENU_ITEMS_RELATED_NAME, label=_("menu items")
        ),
        MultiFieldPanel(
            heading=_("Advanced settings"),
            children=(FieldPanel('max_levels'), FieldPanel('use_specific')),
            classname="collapsible collapsed",
        ),
    )


# ########################################################
# Concrete models
# ########################################################

class MainMenu(AbstractMainMenu):
    """The default model for 'main menu' instances."""
    pass


class FlatMenu(AbstractFlatMenu):
    """The default model for 'flat menu' instances."""
    pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import modelcluster.fields


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0023_alter_page_revision_on_delete_behaviour'),
    ]

    operations = [
        migrations.CreateModel(
            name='FlatMenu',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(help_text='For internal reference only.', max_length=255)),
                ('handle', models.SlugField(help_text='Used in to reference this menu in templates etc. Must be unique for the selected site.', max_length=100)),
                ('heading', models.CharField(help_text='If supplied, appears above the menu when displayed on the on the front-end of the site.', max_length=255, blank=True)),
                ('site', models.ForeignKey(on_delete=models.deletion.CASCADE, related_name='flat_menus', to='wagtailcore.Site', verbose_name='site')),
            ],
            options={
                'verbose_name': 'flat menu',
                'verbose_name_plural': 'flat menus',
            },
        ),
        migrations.CreateModel(
            name='FlatMenuItem',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('sort_order', models.IntegerField(null=True, editable=False, blank=True)),
                ('link_text', models.CharField(help_text="If left blank, the page name will be used. Must be set if you're linking to a custom URL.", max_length=255, blank=True)),
                ('link_url', models.URLField(null=True, verbose_name='Link to a custom URL', blank=True)),
                ('link_page', models.ForeignKey(on_delete=models.deletion.CASCADE, related_name='+', verbose_name='Link to an internal page', blank=True, to='wagtailcore.Page', null=True)),
                ('menu', modelcluster.fields.ParentalKey(on_delete=models.deletion.CASCADE, related_name='menu_items', to='wagtailmenus.FlatMenu')),
            ],
            options={
                'ordering': ['sort_order'],
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MainMenu',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('site', models.OneToOneField(on_delete=models.deletion.CASCADE, related_name='main_menu', to='wagtailcore.Site')),
            ],
            options={
                'verbose_name': 'main menu',
                'verbose_name_plural': 'main menu',
            },
        ),
        migrations.CreateModel(
            name='MainMenuItem',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('sort_order', models.IntegerField(null=True, editable=False, blank=True)),
                ('link_text', models.CharField(help_text="If left blank, the page name will be used. Must be set if you're linking to a custom URL.", max_length=255, blank=True)),
                ('link_url', models.URLField(null=True, verbose_name='Link to a custom URL', blank=True)),
                ('show_children_menu', models.BooleanField(default=True, help_text='The children menu will only appear if this menu item links to a page, and that page has children that are set to appear in menus.', verbose_name='Show a children menu for this item?')),
                ('repeat_in_children_menu', models.BooleanField(help_text="A menu item with children automatically becomes a toggle for accessing the pages below it. Repeating the link in it's children menu allows the page to remain accessible via the main navigation.", verbose_name='Repeat a link to this page in the children menu?')),
                ('children_menu_link_text', models.CharField(help_text='If left blank, the menu item text will be repeated.', max_length=255, verbose_name='Link text for children menu link', blank=True)),
                ('link_page', models.ForeignKey(on_delete=models.deletion.CASCADE, related_name='+', verbose_name='Link to an internal page', blank=True, to='wagtailcore.Page', null=True)),
                ('menu', modelcluster.fields.ParentalKey(on_delete=models.deletion.CASCADE, related_name='menu_items', to='wagtailmenus.MainMenu')),
            ],
            options={
                'ordering': ['sort_order'],
                'abstract': False,
            },
        ),
        migrations.AlterUniqueTogether(
            name='flatmenu',
            unique_together=set([('site', 'handle')]),
        ),
    ]



from __future__ import absolute_import, unicode_literals

from django import forms
from django.utils.translation import ugettext_lazy as _

from wagtail.wagtailadmin.forms import WagtailAdminModelForm, WagtailAdminPageForm

from . import app_settings


class FlatMenuAdminForm(WagtailAdminModelForm):
    def __init__(self, *args, **kwargs):
        super(FlatMenuAdminForm, self).__init__(*args, **kwargs)
        if app_settings.FLAT_MENUS_HANDLE_CHOICES:
            self.fields['handle'] = forms.ChoiceField(
                choices=app_settings.FLAT_MENUS_HANDLE_CHOICES)


class LinkPageAdminForm(WagtailAdminPageForm):
    def __init__(self, *args, **kwargs):
        super(LinkPageAdminForm, self).__init__(*args, **kwargs)
        self.fields['title'].help_text = _(
            "By default, this will be used as the link text when appearing "
            "in menus."
        )



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-18 23:39
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailmenus', '0011_auto_20160415_1519'),
    ]

    operations = [
        migrations.AlterField(
            model_name='flatmenuitem',
            name='link_page',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='wagtailcore.Page', verbose_name='link to an internal page'),
        ),
        migrations.AlterField(
            model_name='flatmenuitem',
            name='link_url',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='link to a custom URL'),
        ),
        migrations.AlterField(
            model_name='flatmenuitem',
            name='url_append',
            field=models.CharField(blank=True, help_text="Use this to optionally append a #hash or querystring to the above page's URL.", max_length=255, verbose_name='append to URL'),
        ),
        migrations.AlterField(
            model_name='mainmenu',
            name='site',
            field=models.OneToOneField(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='main_menu', to='wagtailcore.Site', verbose_name='site'),
        ),
        migrations.AlterField(
            model_name='mainmenuitem',
            name='allow_subnav',
            field=models.BooleanField(default=True, verbose_name='allow sub-navigation for this page'),
        ),
        migrations.AlterField(
            model_name='mainmenuitem',
            name='link_page',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='wagtailcore.Page', verbose_name='link to an internal page'),
        ),
        migrations.AlterField(
            model_name='mainmenuitem',
            name='link_url',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='link to a custom URL'),
        ),
        migrations.AlterField(
            model_name='mainmenuitem',
            name='url_append',
            field=models.CharField(blank=True, help_text="Use this to optionally append a #hash or querystring to the above page's URL.", max_length=255, verbose_name='append to URL'),
        ),
    ]



from __future__ import unicode_literals

from django.conf.urls import url
from django.contrib.admin.utils import quote
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

from wagtail.contrib.modeladmin.options import ModelAdmin, modeladmin_register
from wagtail.contrib.modeladmin.helpers import ButtonHelper
from wagtail.wagtailcore import hooks

from . import app_settings, get_main_menu_model, get_flat_menu_model
from .views import MainMenuIndexView, MainMenuEditView, FlatMenuCopyView


class MainMenuAdmin(ModelAdmin):
    model = get_main_menu_model()
    menu_label = _('Main menu')
    menu_icon = app_settings.MAINMENU_MENU_ICON
    index_view_class = MainMenuIndexView
    edit_view_class = MainMenuEditView
    add_to_settings_menu = True

    def get_form_view_extra_css(self):
        if app_settings.ADD_EDITOR_OVERRIDE_STYLES:
            return ['wagtailmenus/css/menu-edit.css']
        return []

    def get_admin_urls_for_registration(self):
        return (
            url(self.url_helper.get_action_url_pattern('index'),
                self.index_view,
                name=self.url_helper.get_action_url_name('index')),
            url(self.url_helper.get_action_url_pattern('edit'),
                self.edit_view,
                name=self.url_helper.get_action_url_name('edit')),
        )

modeladmin_register(MainMenuAdmin)


class FlatMenuButtonHelper(ButtonHelper):

    def copy_button(self, pk, classnames_add=[], classnames_exclude=[]):
        cn = self.finalise_classname(classnames_add, classnames_exclude)
        return {
            'url': self.url_helper.get_action_url('copy', quote(pk)),
            'label': _('Copy'),
            'classname': cn,
            'title': _('Copy this %(model_name)s') % {
                'model_name': self.verbose_name,
            },
        }

    def get_buttons_for_obj(self, obj, exclude=[], classnames_add=[],
                            classnames_exclude=[]):
        ph = self.permission_helper
        usr = self.request.user
        pk = quote(getattr(obj, self.opts.pk.attname))
        btns = super(FlatMenuButtonHelper, self).get_buttons_for_obj(
            obj, exclude, classnames_add, classnames_exclude)
        if('copy' not in exclude and ph.user_can_create(usr)):
            btns.append(
                self.copy_button(pk, classnames_add, classnames_exclude)
            )
        return btns


class FlatMenuAdmin(ModelAdmin):
    model = get_flat_menu_model()
    menu_label = _('Flat menus')
    menu_icon = app_settings.FLATMENU_MENU_ICON
    button_helper_class = FlatMenuButtonHelper
    ordering = ('-site__is_default_site', 'site__hostname', 'handle')
    add_to_settings_menu = True

    def get_form_view_extra_css(self):
        if app_settings.ADD_EDITOR_OVERRIDE_STYLES:
            return ['wagtailmenus/css/menu-edit.css']
        return []

    def copy_view(self, request, instance_pk):
        kwargs = {'model_admin': self, 'instance_pk': instance_pk}
        return FlatMenuCopyView.as_view(**kwargs)(request)

    def get_admin_urls_for_registration(self):
        urls = super(FlatMenuAdmin, self).get_admin_urls_for_registration()
        urls += (
            url(self.url_helper.get_action_url_pattern('copy'),
                self.copy_view,
                name=self.url_helper.get_action_url_name('copy')),
        )
        return urls

    def get_list_filter(self, request):
        if self.is_multisite_listing(request):
            return ('site', 'handle')
        return ()

    def get_list_display(self, request):
        if self.is_multisite_listing(request):
            return ('title', 'handle_formatted', 'site', 'items')
        return ('title', 'handle_formatted', 'items')

    def handle_formatted(self, obj):
        return mark_safe('<code>%s</code>' % obj.handle)
    handle_formatted.short_description = _('handle')
    handle_formatted.admin_order_field = 'handle'

    def is_multisite_listing(self, request):
        return self.get_queryset(request).values('site').distinct().count() > 1

    def items(self, obj):
        return obj.get_menu_items_manager().count()
    items.short_description = _('no. of items')


modeladmin_register(FlatMenuAdmin)


@hooks.register('before_serve_page')
def wagtailmenu_params_helper(page, request, serve_args, serve_kwargs):
    section_root = request.site.root_page.get_descendants().ancestor_of(
        page, inclusive=True
    ).filter(depth__exact=app_settings.SECTION_ROOT_DEPTH).first()
    if section_root:
        section_root = section_root.specific
    ancestor_ids = page.get_ancestors().filter(
        depth__gte=app_settings.SECTION_ROOT_DEPTH
    ).values_list('id', flat=True)
    request.META.update({
        'WAGTAILMENUS_CURRENT_SECTION_ROOT': section_root,
        'WAGTAILMENUS_CURRENT_PAGE': page,
        'WAGTAILMENUS_CURRENT_PAGE_ANCESTOR_IDS': ancestor_ids,
    })



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-05-17 22:17
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0011_auto_20170106_2355'),
    ]

    operations = [
        migrations.CreateModel(
            name='LinkPage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
                ('link_url', models.CharField(blank=True, max_length=255, null=True, verbose_name='link to a custom URL')),
                ('url_append', models.CharField(blank=True, help_text="Use this to optionally append a #hash or querystring to the above page's URL.", max_length=255, verbose_name='append to URL')),
                ('link_page', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailcore.Page', verbose_name='link to an internal page')),
                ('extra_classes', models.CharField(blank=True, help_text='Optionally specify additional css classes to be added to this page when it appears in menus as a menu item.', max_length=100, verbose_name='menu item css classes')),
            ],
            options={
                'abstract': False,
            },
            bases=('wagtailcore.page',),
        ),
    ]



from __future__ import absolute_import, unicode_literals


class RemovedInWagtailMenus24Warning(DeprecationWarning):
    pass


removed_in_next_version_warning = RemovedInWagtailMenus24Warning


class RemovedInWagtailMenus25Warning(PendingDeprecationWarning):
    pass



# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2016-12-04 20:43
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailmenus', '0017_auto_20161013_1658'),
    ]

    operations = [
        migrations.AlterField(
            model_name='flatmenu',
            name='handle',
            field=models.SlugField(help_text='Used to reference this menu in templates etc. Must be unique for the selected site.', max_length=100, verbose_name='handle'),
        ),
        migrations.AlterField(
            model_name='flatmenu',
            name='heading',
            field=models.CharField(blank=True, help_text='If supplied, appears above the menu when rendered.', max_length=255, verbose_name='heading'),
        ),
        migrations.AlterField(
            model_name='flatmenu',
            name='title',
            field=models.CharField(help_text='For internal reference only.', max_length=255, verbose_name='title'),
        ),
    ]



from django.conf.urls import include, url

from django.views.generic import TemplateView
from wagtail.wagtailadmin import urls as wagtailadmin_urls
from wagtail.wagtailcore import urls as wagtail_urls

urlpatterns = [
    url(r'^admin/', include(wagtailadmin_urls)),
    url(r'^custom-url/$', TemplateView.as_view(template_name='page.html')),
    url(r'^superheroes/marvel-comics/custom-man/about/$',
        TemplateView.as_view(template_name='page.html')),
    url(r'^about-us/meet-the-team/staff-member-one/$',
        TemplateView.as_view(template_name='page.html')),
    url(r'^about-us/meet-the-team/custom-url/$',
        TemplateView.as_view(template_name='page.html')),
    url(r'^news-and-events/$',
        TemplateView.as_view(template_name='page.html')),
    # Hijacking the iron-man page to render a different template, that tests
    # the effect of `use_specific=0` in template tags
    url(r'^superheroes/marvel-comics/iron-man/$',
        TemplateView.as_view(template_name='use-specific-off.html')),
    # Hijacking the batman page to render a different template, that tests
    # the effect of `use_specific=2` in template tags
    url(r'^superheroes/dc-comics/batman/$',
        TemplateView.as_view(template_name='use-specific-top-level.html')),
    # Hijacking the wonder-woman page to render to different template, that
    # tests the effect of `use_specific=3` in template tags
    url(r'^superheroes/dc-comics/wonder-woman/$',
        TemplateView.as_view(template_name='use-specific-always.html')),
    url(r'', include(wagtail_urls)),
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailmenus', '0002_auto_20160129_0901'),
    ]

    operations = [
        migrations.AlterField(
            model_name='flatmenu',
            name='handle',
            field=models.SlugField(help_text='Used to reference this menu in templates etc. Must be unique for the selected site.', max_length=100),
        ),
        migrations.AlterField(
            model_name='flatmenu',
            name='heading',
            field=models.CharField(help_text='If supplied, appears above the menu when rendered.', max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='flatmenuitem',
            name='link_text',
            field=models.CharField(help_text='If left blank, the page title will be used. Must be set if you wish to link to a custom URL.', max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='flatmenuitem',
            name='link_url',
            field=models.CharField(max_length=255, null=True, verbose_name='Link to a custom URL', blank=True),
        ),
        migrations.AlterField(
            model_name='mainmenuitem',
            name='children_menu_link_text',
            field=models.CharField(help_text='e.g. Overview. If left blank, the page title will be used.', max_length=255, verbose_name='Link text for sub-menu item', blank=True),
        ),
        migrations.AlterField(
            model_name='mainmenuitem',
            name='link_text',
            field=models.CharField(help_text='If left blank, the page title will be used. Must be set if you wish to link to a custom URL.', max_length=255, blank=True),
        ),
        migrations.AlterField(
            model_name='mainmenuitem',
            name='link_url',
            field=models.CharField(max_length=255, null=True, verbose_name='Link to a custom URL', blank=True),
        ),
        migrations.AlterField(
            model_name='mainmenuitem',
            name='repeat_in_children_menu',
            field=models.BooleanField(default=True, verbose_name='Include a link to this page in the sub-menu'),
        ),
        migrations.AlterField(
            model_name='mainmenuitem',
            name='show_children_menu',
            field=models.BooleanField(default=True, verbose_name='Add a sub-menu for children of this page'),
        ),
    ]



import os
from setuptools import setup, find_packages
from wagtailmenus import __version__, stable_branch_name

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

base_url = "https://github.com/rkhleics/wagtailmenus/"
dowload_url = '%starball/v%s' % (base_url, __version__)
branch_url = "%stree/stable/%s" % (base_url, stable_branch_name)

# Testing dependencies
testing_extras = [
    'django-webtest>=1.8.0',
    'beautifulsoup4>=4.4.1,<4.5.02',
    'coverage>=3.7.0',
]

documentation_extras = [
    'pyenchant==1.6.6',
    'sphinxcontrib-spelling>=2.3.0',
    'Sphinx>=1.3.1',
    'sphinx-autobuild>=0.5.2',
    'sphinx_rtd_theme>=0.1.8',
]

setup(
    name="wagtailmenus",
    version=__version__,
    author="Andy Babic",
    author_email="ababic@rkh.co.uk",
    description=("An app to help you manage menus in your Wagtail projects "
                 "more consistently."),
    long_description=README,
    packages=find_packages(),
    license="MIT",
    keywords="wagtail cms model utility",
    download_url=dowload_url,
    url=branch_url,
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        'Topic :: Internet :: WWW/HTTP',
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
    ],
    install_requires=[
        "wagtail>=1.5",
    ],
    extras_require={
        'testing': testing_extras,
        'docs': documentation_extras
    },
)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-09-22 08:48
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0028_merge'),
        ('tests', '0007_auto_20160423_1348'),
    ]

    operations = [
        migrations.CreateModel(
            name='ContactPage',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
                ('repeat_in_subnav', models.BooleanField(default=False, help_text="If checked, a link to this page will be repeated alongside it's direct children when displaying a sub-navigation for this page.", verbose_name='repeat in sub-navigation')),
                ('repeated_item_text', models.CharField(blank=True, help_text="e.g. 'Section home' or 'Overview'. If left blank, the page title will be used.", max_length=255, verbose_name='repeated item link text')),
            ],
            options={
                'abstract': False,
            },
            bases=('wagtailcore.page',),
        ),
    ]
